@@ -16,7 +16,6 @@
 # along with Timeline.  If not, see <http://www.gnu.org/licenses/>.
 
 
-from timelinelib.time.numtime import NumTimeType
-from timelinelib.time.pytime import PyTimeType
-from timelinelib.time.wxtime import try_to_create_wx_date_time_from_dmy
-from timelinelib.time.wxtime import WxTimeType
+# Import sometimes otherwise shadowed standard library functions
+from time import time
+from time import clock
