@@ -6,7 +6,7 @@ __license__ = 'GPL v2 or later (details at http://www.gnu.org)'
 # standard library
 import sys
 import sys
-import codecs
+import io
 import re as regex
 import logging
 import os
@@ -174,7 +174,7 @@ class cTagImageEAPnl(wxgTagImageEAPnl.wxgTagImageEAPnl, gmEditArea.cGenericEditA
 		if self.__selected_image_file is not None:
 			try:
 				open(self.__selected_image_file).close()
-			except StandardError:
+			except Exception:
 				valid = False
 				self.__selected_image_file = None
 				gmDispatcher.send(signal = 'statustext', msg = _('Cannot open the image file [%s].') % self.__selected_image_file, beep = True)
@@ -402,6 +402,7 @@ class cImageTagPresenterPnl(wxgVisualSoapPresenterPnl.wxgVisualSoapPresenterPnl)
 		self.__current_tag = evt.GetEventObject().tag
 		self.PopupMenu(self.__context_popup, pos = wx.DefaultPosition)
 		self.__current_tag = None
+
 #============================================================
 #============================================================
 class cKOrganizerSchedulePnl(gmDataMiningWidgets.cPatientListingPnl):
@@ -441,7 +442,7 @@ class cKOrganizerSchedulePnl(gmDataMiningWidgets.cPatientListingPnl):
 		except OSError: pass
 		gmShellAPI.run_command_in_shell(command=self.reload_cmd, blocking=True)
 		try:
-			csv_file = codecs.open(self.fname , mode = 'rU', encoding = 'utf8', errors = 'replace')
+			csv_file = io.open(self.fname , mode = 'rt', encoding = 'utf8', errors = 'replace')
 		except IOError:
 			gmDispatcher.send(signal = u'statustext', msg = _('Cannot access KOrganizer transfer file [%s]') % self.fname, beep = True)
 			return
@@ -474,6 +475,7 @@ class cKOrganizerSchedulePnl(gmDataMiningWidgets.cPatientListingPnl):
 	#--------------------------------------------------------
 	def repopulate_ui(self):
 		self.reload_appointments()
+
 #============================================================
 # occupation related widgets / functions
 #============================================================
@@ -536,6 +538,21 @@ def document_death_of_patient(identity=None):
 
 #------------------------------------------------------------
 def disable_identity(identity=None):
+
+	# already disabled ?
+	if identity['is_deleted']:
+		_log.debug('identity already deleted: %s', identity)
+		return True
+
+	# logged in staff ?
+	# if so -> return
+	prov = gmStaff.gmCurrentProvider()
+	if prov['pk_identity'] == identity['pk_identity']:
+		_log.warning('identity cannot delete itself while being logged on as staff member')
+		_log.debug('identity to delete: %s', identity)
+		_log.debug('logged on staff: %s', prov)
+		return False
+
 	# ask user for assurance
 	go_ahead = gmGuiHelpers.gm_show_question (
 		_('Are you sure you really, positively want\n'
@@ -552,28 +569,32 @@ def disable_identity(identity=None):
 			identity.get_formatted_dob(),
 			gmTools.bool2subst (
 				identity.is_patient,
-				_('This patient DID receive care.'),
-				_('This person did NOT receive care.')
+				_('This patient DID receive care here.'),
+				_('This person did NOT receive care here.')
 			)
 		),
 		_('Disabling person')
 	)
 	if not go_ahead:
-		return True
+		return False
 
 	# get admin connection
 	conn = gmAuthWidgets.get_dbowner_connection (
-		procedure = _('Disabling patient')
+		procedure = _('Disabling person')
 	)
 	# - user cancelled
 	if conn is False:
-		return True
+		return False
 	# - error
 	if conn is None:
-		return False
+		return None
 
-	# now disable patient
-	gmPG2.run_rw_queries(queries = [{'cmd': u"update dem.identity set deleted=True where pk=%s", 'args': [identity['pk_identity']]}])
+	# disable patient
+	gmPerson.disable_identity(identity['pk_identity'])
+
+	# change active patient to logged on staff = myself
+	from Gnumed.wxpython.gmPatSearchWidgets import set_active_patient
+	wx.CallAfter(set_active_patient, patient = prov.identity)
 
 	return True
 
@@ -1054,7 +1075,7 @@ class cPersonNameEAPnl(wxgPersonNameEAPnl.wxgPersonNameEAPnl, gmEditArea.cGeneri
 
 		try:
 			data = kwargs['name']
-			identity = gmPerson.cIdentity(aPK_obj = data['pk_identity'])
+			identity = gmPerson.cPerson(aPK_obj = data['pk_identity'])
 			del kwargs['name']
 		except KeyError:
 			data = None
@@ -1215,10 +1236,10 @@ class cPersonNamesManagerPnl(gmListWidgets.cGenericListManagerPnl):
 
 		gmListWidgets.cGenericListManagerPnl.__init__(self, *args, **kwargs)
 
+		self.refresh_callback = self.refresh
 		self.new_callback = self._add_name
 		self.edit_callback = self._edit_name
 		self.delete_callback = self._del_name
-		self.refresh_callback = self.refresh
 
 		self.__init_ui()
 		self.refresh()
@@ -1330,10 +1351,10 @@ class cPersonIDsManagerPnl(gmListWidgets.cGenericListManagerPnl):
 
 		gmListWidgets.cGenericListManagerPnl.__init__(self, *args, **kwargs)
 
+		self.refresh_callback = self.refresh
 		self.new_callback = self._add_id
 		self.edit_callback = self._edit_id
 		self.delete_callback = self._del_id
-		self.refresh_callback = self.refresh
 
 		self.__init_ui()
 		self.refresh()
@@ -1494,7 +1515,7 @@ class cPersonSocialNetworkManagerPnl(wxgPersonSocialNetworkManagerPnl.wxgPersonS
 
 		self._TCTRL_er_contact.SetValue(gmTools.coalesce(self.__identity['emergency_contact'], u''))
 		if self.__identity['pk_emergency_contact'] is not None:
-			ident = gmPerson.cIdentity(aPK_obj = self.__identity['pk_emergency_contact'])
+			ident = gmPerson.cPerson(aPK_obj = self.__identity['pk_emergency_contact'])
 			self._TCTRL_person.person = ident
 			tt = u'%s\n\n%s\n\n%s' % (
 				tt,
@@ -1565,8 +1586,8 @@ class cPersonSocialNetworkManagerPnl(wxgPersonSocialNetworkManagerPnl.wxgPersonS
 	def _on_button_activate_contact_pressed(self, event):
 		ident = self._TCTRL_person.person
 		if ident is not None:
-			from Gnumed.wxpython import gmPatSearchWidgets
-			gmPatSearchWidgets.set_active_patient(patient = ident, forced_reload = False)
+			from Gnumed.wxpython.gmPatSearchWidgets import set_active_patient
+			wx.CallAfter(set_active_patient, patient = ident)
 
 		event.Skip()
 
@@ -1648,6 +1669,7 @@ class cPersonDemographicsEditorNb(wx.Notebook):
 		self.__identity = identity
 
 	identity = property(_get_identity, _set_identity)
+
 #============================================================
 # old occupation widgets
 #============================================================
@@ -1715,6 +1737,7 @@ class cPatOccupationsPanel(wx.Panel):
 				self.__ident.unlink_occupation(occupation = job['l10n_occupation'])
 			self.__ident.link_occupation(occupation = new_job)
 		return True
+
 #============================================================
 class cNotebookedPatEditionPanel(wx.Panel, gmRegetMixin.cRegetOnPaintMixin):
 	"""Patient demographics plugin for main notebook.
@@ -1767,6 +1790,7 @@ class cNotebookedPatEditionPanel(wx.Panel, gmRegetMixin.cRegetOnPaintMixin):
 			self.__patient_notebook.identity = None
 		self.__patient_notebook.refresh()
 		return True
+
 #============================================================
 #============================================================
 if __name__ == "__main__":
@@ -1816,8 +1840,8 @@ if __name__ == "__main__":
 		if patient is None:
 			print "No patient. Exiting gracefully..."
 			sys.exit(0)
-		from Gnumed.wxpython import gmPatSearchWidgets
-		gmPatSearchWidgets.set_active_patient(patient=patient)
+		from Gnumed.wxpython.gmPatSearchWidgets import set_active_patient
+		set_active_patient(patient = patient)
 		return patient
 	#--------------------------------------------------------
 	if len(sys.argv) > 1 and sys.argv[1] == 'test':
