@@ -28,6 +28,8 @@ from Gnumed.business import gmVaccination
 from Gnumed.business import gmDocuments
 from Gnumed.business import gmProviderInbox
 from Gnumed.business import gmExternalCare
+from Gnumed.business import gmAutoHints
+from Gnumed.business import gmMedication
 
 from Gnumed.wxpython import gmRegetMixin
 from Gnumed.wxpython import gmDemographicsWidgets
@@ -41,6 +43,7 @@ from Gnumed.wxpython import gmVaccWidgets
 from Gnumed.wxpython import gmDocumentWidgets
 from Gnumed.wxpython import gmGuiHelpers
 from Gnumed.wxpython import gmPregWidgets
+from Gnumed.wxpython import gmHabitWidgets
 
 
 _log = logging.getLogger('gm.patient')
@@ -114,10 +117,6 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 	#-----------------------------------------------------
 	# event handling
 	#-----------------------------------------------------
-	# remember to call
-	#	self._schedule_data_reget()
-	# whenever you learn of data changes from database listener
-	# threads, dispatcher signals etc.
 	def __register_interests(self):
 		# client internal signals
 		gmDispatcher.connect(signal = u'pre_patient_unselection', receiver = self._on_pre_patient_unselection)
@@ -210,6 +209,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 		self.__refresh_documents(patient = pat)
 
 		return True
+
 	#-----------------------------------------------------
 	# internal helpers
 	#-----------------------------------------------------
@@ -319,6 +319,11 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 			list_items.append(hint['title'])
 			list_data.append(hint)
 
+		hints = patient.suppressed_hints
+		if len(hints) > 0:
+			list_items.append(_("suppr'd:") + u' ' + u','.join([h['title'][:7] + gmTools.u_ellipsis for h in hints]))
+			list_data.append(_('Suppressed hints:\n') + u'\n'.join([h['title'] for h in hints]))
+
 		self._LCTRL_inbox.set_string_items(items = list_items)
 		self._LCTRL_inbox.set_data(data = list_data)
 
@@ -330,14 +335,18 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 		if isinstance(data, gmProviderInbox.cInboxMessage):
 			return data.format()
 
-		if isinstance(data, gmProviderInbox.cDynamicHint):
-			return u'%s\n\n%s\n\n%s          %s' % (
+		if isinstance(data, gmAutoHints.cDynamicHint):
+			return u'%s\n\n%s%s\n\n%s          %s' % (
 				data['title'],
 				gmTools.wrap(data['hint'], width = 50),
+				gmTools.wrap(gmTools.coalesce(data['recommendation'], u'', u'\n\n%s'), width =  50),
 				gmTools.wrap(gmTools.coalesce(data['url'], u'', u'%s\n\n'), width = 50),
 				data['source']
 			)
 
+		if isinstance(data, type(u'')):
+			return data
+
 		return None
 	#-----------------------------------------------------
 	def _on_inbox_item_activated(self, event):
@@ -345,7 +354,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 		data = self._LCTRL_inbox.get_selected_item_data(only_one = True)
 
 		# if it is a dynamic hint open the URL for that
-		if isinstance(data, gmProviderInbox.cDynamicHint):
+		if isinstance(data, gmAutoHints.cDynamicHint):
 			if data['url'] is not None:
 				gmNetworkTools.open_url_in_browser(data['url'])
 			return
@@ -542,6 +551,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 		self._LCTRL_encounters.set_data(data = list_data)
 		if is_waiting:
 			self._LCTRL_encounters.SetItemTextColour(0, wx.NamedColour('RED'))
+
 	#-----------------------------------------------------
 	def _calc_encounters_list_item_tooltip(self, data):
 		emr = gmPerson.gmCurrentPatient().get_emr()
@@ -667,16 +677,23 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 
 		vaccs = emr.get_latest_vaccinations()
 		for ind, tmp in vaccs.items():
-			tmp, vacc = tmp
-			sort_key = u'%s::%s' % (gmDateTime.pydt_strftime(vacc['date_given'], format = date_format4sorting), vacc['pk_vaccination'])
-			label = _('%s Vacc: %s') % (
+			no_of_shots, vacc = tmp
+			sort_key = u'%s::%s::%s' % (gmDateTime.pydt_strftime(vacc['date_given'], format = date_format4sorting), vacc['pk_vaccination'], ind)
+			label = _('%s Vacc: %s (latest of %s)') % (
 				gmDateTime.pydt_strftime(vacc['date_given'], format = '%Y %b'),
-				ind
+				ind,
+				no_of_shots
 			)
 			sort_key_list.append(sort_key)
 			data[sort_key] = [label, vacc]
 		del vaccs
 
+		for abuse in [ a for a in emr.abused_substances if a['harmful_use_type'] == 3 ]:
+			sort_key = u'%s::%s' % (gmDateTime.pydt_strftime(abuse['last_checked_when'], format = date_format4sorting), abuse['substance'])
+			label = _('Hx of addiction: %s') % abuse['substance']
+			sort_key_list.append(sort_key)
+			data[sort_key] = [label, abuse]
+
 		sort_key_list.sort()
 		sort_key_list.reverse()
 		list_items = []
@@ -703,6 +720,9 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 				with_vaccinations = False
 			).strip(u'\n')
 
+		if isinstance(data, gmMedication.cSubstanceIntakeEntry):
+			return data.format(single_line = False)
+
 		if isinstance(data, gmFamilyHistory.cFamilyHistory):
 			return data.format(include_episode = True, include_comment = True)
 
@@ -710,7 +730,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 			return data.format()
 
 		if isinstance(data, gmEMRStructItems.cPerformedProcedure):
-			return data.format(include_episode = True)
+			return data.format(include_episode = True, include_codes = False, include_address = True, include_comm = True)
 
 		if isinstance(data, gmVaccination.cVaccination):
 			return u'\n'.join(data.format (
@@ -776,13 +796,22 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 	#-----------------------------------------------------
 	#-----------------------------------------------------
 	def __refresh_meds(self, patient=None):
-		# list by brand or substance:
+
 		emr = patient.get_emr()
-		intakes = emr.get_current_substance_intakes(include_inactive = False, include_unapproved = True, order_by = u'substance')
 
 		list_items = []
-		multi_brands_already_seen = []
 		data_items = []
+		first_red = False
+
+		# harmful substance use ?
+		abuses = emr.abused_substances
+		if len([ a for a in abuses if a['harmful_use_type'] in [1, 2] ]) > 0:
+			list_items.append(_('active substance abuse'))
+			data_items.append(u'\n'.join([ a.format(left_margin=0, date_format='%Y %b %d', single_line=True) for a in abuses ]))
+
+		# list by brand or substance:
+		intakes = emr.get_current_medications(include_inactive = False, include_unapproved = True, order_by = u'substance')
+		multi_brands_already_seen = []
 		for intake in intakes:
 			brand = intake.containing_drug
 			if brand is None or len(brand['pk_components']) == 1:
@@ -811,10 +840,17 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 					)
 				))
 				data_items.append(intake)
+
 		self._LCTRL_meds.set_string_items(items = list_items)
 		self._LCTRL_meds.set_data(data = data_items)
+
+		if first_red:
+			self._LCTRL_meds.SetItemTextColour(0, wx.NamedColour('RED'))
+
 	#-----------------------------------------------------
 	def _calc_meds_list_item_tooltip(self, data):
+		if isinstance(data, basestring):
+			return data
 		emr = gmPerson.gmCurrentPatient().get_emr()
 		atcs = []
 		if data['atc_substance'] is not None:
@@ -825,15 +861,22 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 		allg = emr.is_allergic_to(atcs = tuple(atcs), inns = (data['substance'],))
 		if allg is False:
 			allg = None
-		return data.format(one_line = False, allergy = allg, show_all_brand_components = True)
+		return data.format(single_line = False, allergy = allg, show_all_brand_components = True)
 	#-----------------------------------------------------
 	def _on_meds_item_activated(self, event):
 		data = self._LCTRL_meds.get_selected_item_data(only_one = True)
-		if data is not None:
-			# <ctrl> down ?
-			if wx.GetKeyState(wx.WXK_CONTROL):
-				wx.CallAfter(gmMedicationWidgets.edit_intake_of_substance, parent = self, substance = data)
-				return
+
+		if data is None:
+			return
+
+#		if isinstance(data, basestring):
+#			gmHabitWidgets.manage_substance_abuse(parent = self, patient = gmPerson.gmCurrentPatient())
+#			return
+
+		# <ctrl> down ? -> edit
+		if wx.GetKeyState(wx.WXK_CONTROL):
+			wx.CallAfter(gmMedicationWidgets.edit_intake_of_substance, parent = self, substance = data)
+			return
 
 		gmDispatcher.send(signal = 'display_widget', name = 'gmCurrentSubstancesPlugin')
 	#-----------------------------------------------------
@@ -876,11 +919,10 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 
 		provider = patient.primary_provider
 		if provider is not None:
-			list_items.append(_('in-praxis: %s') % provider.identity['description_gender'])
+			list_items.append(_('in-praxis: %s') % patient.primary_provider_identity['description_gender'])
 			list_data.append(provider)
 
-		care = emr.get_external_care_items()
-		for item in care:
+		for item in emr.external_care_items:
 			list_items.append(_('care: %s%s@%s') % (
 				gmTools.coalesce(item['provider'], u'', u'%s, '),
 				item['unit'],
@@ -916,7 +958,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 				parts.append(data['comment'])
 			return u'\n'.join(parts)
 
-		if isinstance(data, gmPerson.cIdentity):
+		if isinstance(data, gmPerson.cPerson):
 			return u'%s\n\n%s' % (
 				data['description_gender'],
 				u'\n'.join([
@@ -963,7 +1005,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 				if isinstance(data, gmDemographicRecord.cCommChannel):
 					gmContactWidgets.edit_comm_channel(parent = self, comm_channel = data, channel_owner = gmPerson.gmCurrentPatient())
 					return
-				if isinstance(data, gmPerson.cIdentity):
+				if isinstance(data, gmPerson.cPerson):
 					pass
 				if isinstance(data, gmStaff.cStaff):
 					pass
@@ -1017,7 +1059,11 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 	def _calc_problem_list_item_tooltip(self, data):
 
 		if isinstance(data, gmExternalCare.cExternalCareItem):
-			return u'\n'.join(data.format(with_health_issue = True, with_comms = True))
+			return u'\n'.join(data.format (
+				with_health_issue = True,
+				with_address = True,
+				with_comms = True
+			))
 
 		emr = gmPerson.gmCurrentPatient().get_emr()
 
