@@ -323,7 +323,7 @@ class cInboxMessageEAPnl(wxgInboxMessageEAPnl.wxgInboxMessageEAPnl, gmEditArea.c
 				if self.data['pk_patient'] is None:
 					self._PRW_patient.person = None
 				else:
-					self._PRW_patient.person = gmPerson.cIdentity(aPK_obj = self.data['pk_patient'])
+					self._PRW_patient.person = gmPerson.cPerson(aPK_obj = self.data['pk_patient'])
 		else:
 			self._CHBOX_active_patient.Enable(False)
 			self._CHBOX_active_patient.SetValue(False)
@@ -331,7 +331,7 @@ class cInboxMessageEAPnl(wxgInboxMessageEAPnl.wxgInboxMessageEAPnl, gmEditArea.c
 			if self.data['pk_patient'] is None:
 				self._PRW_patient.person = None
 			else:
-				self._PRW_patient.person = gmPerson.cIdentity(aPK_obj = self.data['pk_patient'])
+				self._PRW_patient.person = gmPerson.cPerson(aPK_obj = self.data['pk_patient'])
 
 		self._PRW_due.SetText(data = self.data['due_date'])
 		self._PRW_expiry.SetText(data = self.data['expiry_date'])
@@ -480,6 +480,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 		self._LCTRL_provider_inbox.set_columns([u'', _('Generated'), _('Status'), _('Category - Type'), _('Message')])
 		self._LCTRL_provider_inbox.searchable_columns = [2, 3, 4]
 		self._LCTRL_provider_inbox.item_tooltip_callback = self._get_msg_tooltip
+		self._LCTRL_provider_inbox.extend_popup_menu_callback = self._extend_popup_menu
 
 		self.__update_greeting()
 
@@ -544,7 +545,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 				order_by = u'due_date, importance DESC, received_when DESC'
 			)
 		elif self._RBTN_scheduled_messages.GetValue():
-			_log.debug('loading overdue messages only')
+			_log.debug('loading scheduled messages only')
 			self.__msgs = self.provider.inbox.get_messages (
 				pk_patient = pk_patient,
 				include_without_provider = include_without_provider,
@@ -587,7 +588,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 			item.append(u'%s: %s%s%s' % (
 				gmDateTime.pydt_strftime(m['received_when'], '%Y-%m-%d'),
 				m['modified_by'],
-				gmTools.u_right_arrow,
+				gmTools.u_arrow2right,
 				gmTools.coalesce(m['provider'], _(u'all'))
 			))
 			if m['due_date'] is None:
@@ -695,29 +696,21 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 
 		self._TXT_inbox_item_comment.SetValue(tmp)
 	#--------------------------------------------------------
-	def _lst_item_right_clicked(self, evt):
+	def _extend_popup_menu(self, menu=None):
 		tmp = self._LCTRL_provider_inbox.get_selected_item_data(only_one = True)
 		if tmp is None:
 			return
 		self.__focussed_msg = tmp
 
-		# build menu
-		menu = wx.Menu(title = _('Inbox Message Actions:'))
-
 		if self.__focussed_msg['pk_patient'] is not None:
-			ID = wx.NewId()
-			menu.AppendItem(wx.MenuItem(menu, ID, _('Activate patient')))
-			wx.EVT_MENU(menu, ID, self._on_goto_patient)
+			menu_item = menu.Append(-1, _('Activate patient'))
+			self.Bind(wx.EVT_MENU, self._on_goto_patient, menu_item)
 
 		if not self.__focussed_msg['is_virtual']:
-			# - delete message
-			ID = wx.NewId()
-			menu.AppendItem(wx.MenuItem(menu, ID, _('Delete')))
-			wx.EVT_MENU(menu, ID, self._on_delete_focussed_msg)
-			# - edit message
-			ID = wx.NewId()
-			menu.AppendItem(wx.MenuItem(menu, ID, _('Edit')))
-			wx.EVT_MENU(menu, ID, self._on_edit_focussed_msg)
+			menu_item = menu.Append(-1, _('Delete'))
+			self.Bind(wx.EVT_MENU, self._on_delete_focussed_msg, menu_item)
+			menu_item = menu.Append(-1, _('Edit'))
+			self.Bind(wx.EVT_MENU, self._on_edit_focussed_msg, menu_item)
 
 #		if self.__focussed_msg['pk_staff'] is not None:
 #			# - distribute to other providers
@@ -725,9 +718,6 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 #			menu.AppendItem(wx.MenuItem(menu, ID, _('Distribute')))
 #			wx.EVT_MENU(menu, ID, self._on_distribute_focussed_msg)
 
-		# show menu
-		self.PopupMenu(menu, wx.DefaultPosition)
-		menu.Destroy()
 	#--------------------------------------------------------
 	def _on_message_range_radiobutton_selected(self, event):
 		self._TXT_inbox_item_comment.SetValue(u'')
@@ -769,7 +759,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 		if pk_patient is not None:
 			#emr = gmClinicalRecord.cClinicalRecord(aPKey = pk_patient, allow_user_interaction = False)
 			from Gnumed.wxpython import gmChartPullingWidgets
-			emr = gmChartPullingWidgets.pull_chart(gmPerson.cIdentity(pk_patient))
+			emr = gmChartPullingWidgets.pull_chart(gmPerson.cPerson(pk_patient))
 			if emr is None:
 				return False
 			epi = emr.add_episode(episode_name = 'administrative', is_open = False)
@@ -812,7 +802,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 		) % pk_patient
 
 		try:
-			pat = gmPerson.cIdentity(aPK_obj = pk_patient)
+			pat = gmPerson.cPerson(aPK_obj = pk_patient)
 		except gmExceptions.ConstructorError:
 			wx.EndBusyCursor()
 			_log.exception('patient [%s] not found', pk_patient)
@@ -842,7 +832,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 		wx.BeginBusyCursor()
 
 		try:
-			pat = gmPerson.cIdentity(aPK_obj = pk_patient)
+			pat = gmPerson.cPerson(aPK_obj = pk_patient)
 		except gmExceptions.ConstructorError:
 			wx.EndBusyCursor()
 			_log.exception('patient [%s] not found', pk_patient)
@@ -870,7 +860,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 		wx.BeginBusyCursor()
 
 		try:
-			pat = gmPerson.cIdentity(aPK_obj = pk_patient)
+			pat = gmPerson.cPerson(aPK_obj = pk_patient)
 		except gmExceptions.ConstructorError:
 			wx.EndBusyCursor()
 			_log.exception('patient [%s] not found', pk_patient)
@@ -898,7 +888,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 		wx.BeginBusyCursor()
 
 		try:
-			pat = gmPerson.cIdentity(aPK_obj = pk_patient)
+			pat = gmPerson.cPerson(aPK_obj = pk_patient)
 		except gmExceptions.ConstructorError:
 			wx.EndBusyCursor()
 			_log.exception('patient [%s] not found', pk_patient)
